@@ -26,27 +26,50 @@
 
 from kiwi.component import Interface
 
-Nothing = object()
+class IProxyWidget(Interface):
+    """
+    IProxyWidget is a widget that can be attached to a proxy.
 
-class Mixin(Interface):
-    # gsignal('content-changed')
-    # gproperty('data-type')
-    # gproperty('model-attribute')
-    # gproperty('validation-changed')
+    Signals::
+       content-changed: This must be emitted when the content changes
 
-    def read(self):
+    Properties::
+       data-type: string, data type of the model
+       model-attribute: string, name of the attribute in the model
+    """
+
+    def read():
+        """
+        Reads the content of the widget and returns in an appropriate
+        data type.
+        ValueUnset is returned when the user has not modified the entry
+        """
         pass
 
-    def update(self, value):
+    def update(value):
+        """
+        Updates the content of the widget with a value
+        """
         pass
 
-class MixinSupportValidation(Interface):
-    # gproperty('mandatory')
-    def is_valid(self):
+class IValidatableProxyWidget(IProxyWidget):
+    """
+    IValidatableProxyWidget extends IProxyWidget with validation support
+
+    Signals::
+       validate: This emitted so each widget can provide it's own
+         custom validation.
+       validation-changed: This is emitted when the validation status
+         changes, mainly used by the proxy.
+
+    Properties::
+       mandatory: bool, if the widget is mandatory
+    """
+
+    def is_valid():
         pass
 
-    # These two are mainly used by subclasses
-    def validate(self, force=False):
+    def validate(force=False):
         pass
 
 class AbstractGladeAdaptor(Interface):
