@@ -33,6 +33,7 @@ import pango
 import gtk
 
 from kiwi.ui.icon import IconEntry
+from kiwi.ui.entrycompletion import KiwiEntryCompletion
 from kiwi.utils import PropertyObject, gsignal, gproperty, type_register
 
 class MaskError(Exception):
@@ -67,8 +68,9 @@ INPUT_CHAR_MAP = {
 (COL_TEXT,
  COL_OBJECT) = range(2)
 
-(ENTRY_MODE_TEXT,
- ENTRY_MODE_DATA) = range(2)
+(ENTRY_MODE_UNKNOWN,
+ ENTRY_MODE_TEXT,
+ ENTRY_MODE_DATA) = range(3)
 
 _ = lambda msg: gettext.dgettext('kiwi', msg)
 
@@ -88,6 +90,8 @@ class KiwiEntry(PropertyObject, gtk.Entry):
     gproperty("mask", str, default='')
 
     def __init__(self):
+        self._completion = None
+
         gtk.Entry.__init__(self)
         PropertyObject.__init__(self)
 
@@ -150,6 +154,17 @@ class KiwiEntry(PropertyObject, gtk.Entry):
         return ''
 
     # Public API
+    def set_text(self, text):
+        completion = self.get_completion()
+
+        if isinstance(completion, KiwiEntryCompletion):
+            self.handler_block(completion.changed_id)
+
+        gtk.Entry.set_text(self, text)
+
+        if isinstance(completion, KiwiEntryCompletion):
+            self.handler_unblock(completion.changed_id)
+
     def set_mask(self, mask):
         """
         Sets the mask of the Entry.
@@ -169,12 +184,12 @@ class KiwiEntry(PropertyObject, gtk.Entry):
         @param mask: the mask to set
         """
 
-        mask = unicode(mask)
         if not mask:
             self.modify_font(pango.FontDescription("sans"))
             self._mask = mask
             return
 
+        mask = unicode(mask)
         input_length = len(mask)
         lenght = 0
         pos = 0
@@ -364,8 +379,32 @@ class KiwiEntry(PropertyObject, gtk.Entry):
         self.set_completion(completion)
         return completion
 
+    def get_completion(self):
+        return self._completion
+
     def set_completion(self, completion):
-        gtk.Entry.set_completion(self, completion)
+        if not isinstance(completion, KiwiEntryCompletion):
+            gtk.Entry.set_completion(self, completion)
+            completion.set_model(gtk.ListStore(str, object))
+            completion.set_text_column(0)
+            self._completion = gtk.Entry.get_completion(self)
+            return
+
+        old = self.get_completion()
+        if old == completion:
+            return completion
+
+        if old and isinstance(old, KiwiEntryCompletion):
+            if old.completion_timeout:
+                gobject.source_remove(old.completion_timeout)
+                old.completion_timeout = 0
+
+            old._disconnect_completion_signals()
+
+        self._completion = completion
+
+        # First, tell the completion what entry it will complete
+        completion.set_entry(self)
         completion.set_model(gtk.ListStore(str, object))
         completion.set_text_column(0)
         self.set_exact_completion(False)
@@ -374,21 +413,21 @@ class KiwiEntry(PropertyObject, gtk.Entry):
         self._current_object = None
         return completion
 
-    def _completion_exact_match_func(self, completion, _, iter):
+    def _completion_exact_match_func(self, completion, key, iter):
         model = completion.get_model()
         if not len(model):
             return
 
         content = model[iter][COL_TEXT]
-        return self.get_text().startswith(content)
+        return key.startswith(content)
 
-    def _completion_normal_match_func(self, completion, _, iter):
+    def _completion_normal_match_func(self, completion, key, iter):
         model = completion.get_model()
         if not len(model):
             return
 
         content = model[iter][COL_TEXT].lower()
-        return self.get_text().lower() in content
+        return key.lower() in content
 
     def _on_completion__match_selected(self, completion, model, iter):
         if not len(model):
@@ -451,6 +490,9 @@ class KiwiEntry(PropertyObject, gtk.Entry):
     def update_background(self, color):
         self._icon.update_background(color)
 
+    def get_background(self):
+        return self._icon.get_background()
+
     def get_icon_window(self):
         return self._icon.get_icon_window()
 
@@ -500,7 +542,7 @@ class KiwiEntry(PropertyObject, gtk.Entry):
             for item in itemdata:
                 if item in values:
                     raise KeyError("Tried to insert duplicate value "
-                                   "%s into Combo!" % item)
+                                   "%r into the entry" % item)
                 else:
                     values[item] = None
 
@@ -513,7 +555,7 @@ class KiwiEntry(PropertyObject, gtk.Entry):
                 text, data = item
                 if text in values:
                     raise KeyError("Tried to insert duplicate value "
-                                   "%s into Combo!" % item)
+                                   "%r into the entry" % text)
                 else:
                     values[text] = None
                 model.append((text, data))
@@ -578,6 +620,9 @@ class KiwiEntry(PropertyObject, gtk.Entry):
             #      data mode
             raise TypeError("unknown Entry mode. Did you call prefill?")
 
+    def get_mode(self):
+        return self._mode
+
 type_register(KiwiEntry)
 
 def main(args):
