@@ -24,15 +24,19 @@
 
 """Gazpacho integration: loader and extensions"""
 
-import datetime
-from decimal import Decimal
 import gettext
 import os
 import warnings
 
 import gobject
 import gtk
-from gazpacho.editor import PropertyCustomEditor
+
+try:
+    from gazpacho.propertyeditor import PropertyCustomEditor
+    PropertyCustomEditor # pyflakes
+except ImportError:
+    from gazpacho.editor import PropertyCustomEditor
+
 from gazpacho.loader.loader import ObjectBuilder
 from gazpacho.loader.custom import Adapter, ComboBoxAdapter, \
      PythonWidgetAdapter, adapter_registry
@@ -40,12 +44,13 @@ from gazpacho.properties import prop_registry, CustomProperty, StringType
 from gazpacho.widgets.base.base import ContainerAdaptor
 from gazpacho.widgets.base.box import BoxAdaptor
 
-from kiwi.datatypes import currency
+from kiwi.datatypes import converter
 from kiwi.environ import environ
 from kiwi.log import Logger
 from kiwi.python import disabledeprecationcall
 from kiwi.ui.hyperlink import HyperLink
 from kiwi.ui.objectlist import Column, ObjectList
+from kiwi.ui.widgets.button import ProxyButton
 from kiwi.ui.widgets.checkbutton import ProxyCheckButton
 from kiwi.ui.widgets.combo import ProxyComboEntry, ProxyComboBox, \
      ProxyComboBoxEntry
@@ -59,7 +64,7 @@ from kiwi.ui.widgets.textview import ProxyTextView
 from kiwi.ui.widgets.combobox import ComboBox, ComboBoxEntry
 from kiwi.ui.widgets.list import List
 HyperLink
-_ = gettext.gettext
+_ = lambda m: gettext.dgettext('kiwi', m)
 
 log = Logger('gazpacholoader')
 
@@ -165,26 +170,26 @@ adapter_registry.register_adapter(ObjectListAdapter)
 # Framework widgets
 
 class DataTypeAdaptor(PropertyCustomEditor):
+    widget_type = None
+
     def __init__(self):
         super(DataTypeAdaptor, self).__init__()
+        self._model = None
         self._input = self.create_editor()
 
     def get_editor_widget(self):
         return self._input
 
-    def get_data_types(self):
-        """
-        Subclasses should override this.
-        Expected to return a list of 2 sized tuples with
-        name of type and type, to be used in a combo box.
-        """
-        raise NotImplementedError
+    def _get_converters(self):
+        if not self.widget_type:
+            AssertionError("%r should define a widget_type" % self)
+
+        allowed = self.widget_type.allowed_data_types
+        return converter.get_converters(allowed)
 
     def create_editor(self):
-        model = gtk.ListStore(str, object)
-        for datatype in self.get_data_types():
-            model.append(datatype)
-        combo = gtk.ComboBox(model)
+        self._model = gtk.ListStore(str, object)
+        combo = gtk.ComboBox(self._model)
         renderer = gtk.CellRendererText()
         combo.pack_start(renderer)
         combo.add_attribute(renderer, 'text', 0)
@@ -194,6 +199,11 @@ class DataTypeAdaptor(PropertyCustomEditor):
 
     def update(self, context, kiwiwidget, proxy):
         combo = self._input
+        model = self._model
+        model.clear()
+        for converter in self._get_converters():
+            model.append((converter.name, converter.type))
+
         connection_id = combo.get_data('connection-id')
         if (connection_id != -1):
             combo.disconnect(connection_id)
@@ -212,66 +222,28 @@ class DataTypeAdaptor(PropertyCustomEditor):
         proxy.set_value(model[active_iter][1])
 
 class SpinBtnDataType(DataTypeAdaptor):
-    def get_data_types(self):
-        return [
-            (_('Integer'), int),
-            (_('Float'), float),
-            (_('Decimal'), Decimal),
-            (_('Currency'), currency)
-            ]
+    widget_type = ProxySpinButton
 
 class EntryDataType(DataTypeAdaptor):
-    def get_data_types(self):
-        return [
-            (_('String'), str),
-            (_('Unicode'), unicode),
-            (_('Integer'), int),
-            (_('Float'), float),
-            (_('Decimal'), Decimal),
-            (_('Currency'), currency),
-            (_('Date'), datetime.date),
-            (_('Date and Time'), datetime.datetime),
-            (_('Time'), datetime.time),
-            (_('Object'), object)
-            ]
+    widget_type = ProxyEntry
 
 class TextViewDataType(DataTypeAdaptor):
-    def get_data_types(self):
-        return [
-            (_('String'), str),
-            (_('Unicode'), unicode),
-            (_('Integer'), int),
-            (_('Float'), float),
-            (_('Decimal'), Decimal),
-            (_('Date'), datetime.date),
-            ]
+    widget_type = ProxyTextView
 
 class ComboBoxDataType(DataTypeAdaptor):
-    def get_data_types(self):
-        return [
-            (_('String'), str),
-            (_('Unicode'), unicode),
-            (_('Boolean'), bool),
-            (_('Integer'), int),
-            (_('Float'), float),
-            (_('Decimal'), Decimal),
-            (_('Object'), object)
-            ]
+    widget_type = ProxyComboBox
+
+class ComboBoxEntryDataType(DataTypeAdaptor):
+    widget_type = ProxyComboBoxEntry
+
+class ComboEntryDataType(DataTypeAdaptor):
+    widget_type = ProxyComboEntry
 
 class LabelDataType(DataTypeAdaptor):
-    def get_data_types(self):
-        return [
-            (_('String'), str),
-            (_('Unicode'), unicode),
-            (_('Boolean'), bool),
-            (_('Integer'), int),
-            (_('Float'), float),
-            (_('Decimal'), Decimal),
-            (_('Date'), datetime.date),
-            (_('Date and Time'), datetime.datetime),
-            (_('Time'), datetime.time),
-            (_('Currency'), currency)
-            ]
+    widget_type = ProxyLabel
+
+class ButtonDataType(DataTypeAdaptor):
+    widget_type = ProxyButton
 
 class DataType(CustomProperty, StringType):
     translatable = False
@@ -322,11 +294,12 @@ def register_widgets():
     for gobj, editor, data_type in [
         (ProxyEntry, EntryDataType, DataType),
         (ProxyDateEntry, None, DateOnlyDataType),
+        (ProxyButton, ButtonDataType, DataType),
         (ProxyCheckButton, None, BoolOnlyDataType),
         (ProxyLabel, LabelDataType, DataType),
         (ProxyComboBox, ComboBoxDataType, DataType),
-        (ProxyComboBoxEntry, ComboBoxDataType, DataType),
-        (ProxyComboEntry, ComboBoxDataType, DataType),
+        (ProxyComboBoxEntry, ComboBoxEntryDataType, DataType),
+        (ProxyComboEntry, ComboEntryDataType, DataType),
         (ProxySpinButton, SpinBtnDataType, DataType),
         (ProxyRadioButton, None, BoolOnlyDataType),
         (ProxyTextView, TextViewDataType, DataType)
@@ -336,7 +309,7 @@ def register_widgets():
 
         data_name = type_name + '::data-type'
         if editor:
-            prop_registry.override_simple(data_name, data_type, editor=editor)
+            prop_registry.override_simple(data_name, data_type, custom_editor=editor)
         else:
             prop_registry.override_simple(data_name, data_type)
 
