@@ -28,16 +28,19 @@
 
 import gtk
 
-from kiwi.ui.widgets.proxy import WidgetMixin
-from kiwi.utils import PropertyObject, gsignal
+from kiwi.python import deprecationwarn
+from kiwi.ui.proxywidget import ProxyWidgetMixin
+from kiwi.utils import PropertyObject, gsignal, type_register
+
+class ProxyCheckButton(PropertyObject, gtk.CheckButton, ProxyWidgetMixin):
+    __gtype_name__ = 'ProxyCheckButton'
 
-class CheckButton(PropertyObject, gtk.CheckButton, WidgetMixin):
     # changed allowed data types because checkbuttons can only
     # accept bool values
     allowed_data_types = bool,
 
     def __init__(self):
-        WidgetMixin.__init__(self)
+        ProxyWidgetMixin.__init__(self)
         PropertyObject.__init__(self, data_type=bool)
         gtk.CheckButton.__init__(self)
 
@@ -51,7 +54,16 @@ class CheckButton(PropertyObject, gtk.CheckButton, WidgetMixin):
 
     def update(self, data):
         if data is None:
+            self.set_active(False);
             return
 
         # No conversion to string needed, we only accept bool
         self.set_active(data)
+
+class CheckButton(ProxyCheckButton):
+    def __init__(self):
+        deprecationwarn(
+            'CheckButton is deprecated, use ProxyCheckButton instead',
+            stacklevel=3)
+        ProxyCheckButton.__init__(self)
+type_register(CheckButton)
